package com.anavrinapps.flutter_mdata;

import androidx.annotation.NonNull;
import android.app.Activity;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.BinaryMessenger;

import io.monedata.*;
import kotlin.Unit;

public class FlutterMdataPlugin implements FlutterPlugin, MethodCallHandler,ActivityAware {
  private MethodChannel channel;
  static Activity activity;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "monedatasdk");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("init")) {
      String api_token = null;
  if(call.argument("api_token")!=null) {
    api_token = call.argument("api_token");
  }else{
    result.error("no_api_token", "a null api token was provided", null);
    return;
  }
      Monedata.initialize(activity.getApplicationContext(), api_token, true, isReady -> {
        try {
     FlutterMdataPlugin.activity.runOnUiThread(new Runnable() {
               @Override
               public void run() {
                   channel.invokeMethod("isReady",isReady);
               }
           });
       } catch (Exception e) {
           Log.e("Monedata", "Error " + e.toString());
    }
        return Unit.INSTANCE;
    });
    } else if(call.method.equals("start")) {
      Monedata.start(activity.getApplicationContext());
    } else if(call.method.equals("stop")) {
      Monedata.stop(activity.getApplicationContext());
    }  else if(call.method.equals("request")) {
    Monedata.Consent.request(activity, false, consent -> {
       try {
     FlutterMdataPlugin.activity.runOnUiThread(new Runnable() {
               @Override
               public void run() {
                   channel.invokeMethod("didRequest",consent);
               }
           });
       } catch (Exception e) {
           Log.e("Monedata", "Error " + e.toString());
    }
        return Unit.INSTANCE;
    });
    } else if(call.method.equals("optOutPopup")) {
      Monedata.Consent.request(activity, true);
    } else if(call.method.equals("addListener")) {
      Monedata.Consent.addListener(consent -> {
        try {
     FlutterMdataPlugin.activity.runOnUiThread(new Runnable() {
               @Override
               public void run() {
                   channel.invokeMethod("didAddListener",consent);
               }
           });
       } catch (Exception e) {
           Log.e("Monedata", "Error " + e.toString());
    }
        return Unit.INSTANCE;    
    });
    } else if(call.method.equals("removeListener")) {
      Monedata.Consent.removeListener(consent -> {
         try {
     FlutterMdataPlugin.activity.runOnUiThread(new Runnable() {
               @Override
               public void run() {
                   channel.invokeMethod("didRemoveListener",consent);
               }
           });
       } catch (Exception e) {
           Log.e("Monedata", "Error " + e.toString());
    }
        return Unit.INSTANCE;  
    });
    }
    else {
      result.notImplemented();
    }
  }

 @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding activityPluginBinding) {
    activity = activityPluginBinding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding activityPluginBinding) {
  }

  @Override
  public void onDetachedFromActivity() {
  }
}
